@@ -8,7 +8,9 @@ PyAlaCarte and PyAlaMode editors from wxPython
 Combines the lab_bench and offers the possibility to transform the
 current work into script files.
 """
+
 import sys
+import os
 
 import wx
 import wx.lib.dialogs
@@ -19,11 +21,43 @@ import wx.py.dispatcher as dispatcher
 
 from wx.py.editor import *
 
-from labbench import LabBench
-import advisor
+from mathbench.basement.history_manager import HistoryManager
+
+from mathbench.lab.labbench import LabBench
+import mathbench.lab.advisor as advisor
 
 from mathbench.basement.librarian import LibrarianSingleton
 
+ID_EXECUTE = wx.NewId()
+ID_EXECUTE_MAIN = wx.NewId()
+ID_PREFERENCE = wx.NewId()
+ID_LIBRARY = wx.NewId()
+ID_HISTORY_TO_SCRIPT = wx.NewId()
+ID_LOAD_OLD_SESSION = wx.NewId()
+
+
+from wx.html import HtmlEasyPrinting
+
+class Printer(HtmlEasyPrinting):
+	"""
+	Small printing class taken from: http://wiki.wxpython.org/Printing
+	"""
+	def __init__(self):
+		HtmlEasyPrinting.__init__(self)
+
+	def GetHtmlText(self,text):
+		"Simple conversion of text.	 Use a more powerful version"
+		html_text = text.replace('\n', '<BR>')
+		return html_text
+
+	def Print(self, text, doc_name):
+		self.SetHeader(doc_name)
+		self.PrintText(self.GetHtmlText(text),doc_name)
+
+	def PreviewText(self, text, doc_name):
+		self.SetHeader(doc_name)
+		HtmlEasyPrinting.PreviewText(self, self.GetHtmlText(text))
+
 class LabEditorNotebook(EditorNotebook):
 	"""
 	Very light customisation of the base class
@@ -62,12 +96,14 @@ class LabBook(EditorNotebookFrame):
 	_init_shell_session = []
 
 
-	def __init__(self, parent=None, id=-1, title='Math Bench', config=None,
+	def __init__(self, parent=None, id=-1, title='MathBench', 
+				 config=None,
 				 pos=wx.DefaultPosition, size=(800, 600), 
 				 style=wx.DEFAULT_FRAME_STYLE | wx.NO_FULL_REPAINT_ON_RESIZE,
-				 filename=None):
+				 filename=None, history_dir=None):
 		"""Create LabBookFrame instance."""
 		self.config = config
+		self.history_dir = history_dir
 		EditorNotebookFrame.__init__(self, parent, id, title, pos,
 									 size, style, filename)
 		self._modify_menus()
@@ -75,12 +111,11 @@ class LabBook(EditorNotebookFrame):
 		self.options_frame = None
 		# keep track of the shells opened for the exectution of a spectific file
 		self.execution_shells = {}
-		
 
 	def __new__(cls,parent=None, id=-1, title='Math Bench', config=None,
 				 pos=wx.DefaultPosition, size=(800, 600), 
 				 style=wx.DEFAULT_FRAME_STYLE | wx.NO_FULL_REPAINT_ON_RESIZE,
-				 filename=None):
+				 filename=None, history_dir=None):
 		"""
 		Make sure only one instance is ever created
 		"""
@@ -107,10 +142,11 @@ class LabBook(EditorNotebookFrame):
 		help_sizer = wx.BoxSizer(wx.HORIZONTAL)
 		tb_img_size = (16,16)
 		bt_size = (20,20)
+		art_prov = wx.ArtProvider()
 		# show library
-		img = wx.ArtProvider_GetBitmap(wx.ART_HELP, 
-									   wx.ART_TOOLBAR, 
-									   tb_img_size)
+		img = art_prov.GetBitmap(wx.ART_HELP_BOOK, 
+								 wx.ART_TOOLBAR, 
+								 tb_img_size)
 		btn = wx.BitmapButton(self.helpbar, 50, img)
 		btn.SetToolTipString("Show the library desk.")
 		self.Bind(wx.EVT_BUTTON, self.OnHelpToolClick, id=50)
@@ -129,9 +165,9 @@ class LabBook(EditorNotebookFrame):
 		help_sizer.Add(self.query_widget,proportion=1,flag=wx.EXPAND | wx.ALL, border=3)
 
 		# delete button
-		img = wx.ArtProvider_GetBitmap(wx.ART_DELETE, 
-									   wx.ART_TOOLBAR, 
-									   tb_img_size)
+		img = art_prov.GetBitmap(wx.ART_DELETE, 
+								 wx.ART_TOOLBAR, 
+								 tb_img_size)
 		btn = wx.BitmapButton(self.helpbar, 30, img)
 		btn.SetToolTipString("Clear the search form.")
 		self.Bind(wx.EVT_BUTTON, self.OnHelpToolClick, id=30)
@@ -139,9 +175,9 @@ class LabBook(EditorNotebookFrame):
 		self.helpbar.SetSizer(help_sizer)
 
 		# search button
-		img = wx.ArtProvider_GetBitmap(wx.ART_FIND, 
-									   wx.ART_TOOLBAR, 
-									   tb_img_size)
+		img = art_prov.GetBitmap(wx.ART_FIND, 
+								 wx.ART_TOOLBAR, 
+								 tb_img_size)
 		btn = wx.BitmapButton(self.helpbar,35, img)
 		btn.SetToolTipString("Search the documentation.")
 		self.Bind(wx.EVT_BUTTON, self.OnHelpToolClick, id=35)
@@ -161,7 +197,13 @@ class LabBook(EditorNotebookFrame):
 		intro = "Welcome in MathBench"
 		self.labBench = LabBench(parent=self.notebook, intro=intro, locals=namespace)
 		self.shell = self.labBench.shell
-		
+		# if a directory to put the session files in is provided, then
+		# instanciate a History manager.
+		if self.history_dir is not None:
+			self.historyman = HistoryManager(self.shell, self.history_dir)	
+			dispatcher.connect(receiver=self.historyman.save_current_session, signal='LabBook.close')
+			self.historyman.load_latest_session()
+			self.historyman.put_landmark()
 		
 	def _setup(self):
 		"""
@@ -179,10 +221,10 @@ class LabBook(EditorNotebookFrame):
 		self.SetSizer(sizer)
 		# create the shell
 		self._init_shell_widget()
-		# Override the logBook so that status messages go to the status bar.
-		self.labBench.logBook.setStatusText = self.SetStatusText
-		# And tell the logBook to use our sink when the user wants to extract the log
-		self.labBench.logBook.log_sink = self.log_sink_to_file
+# 		# Override the logBook so that status messages go to the status bar.
+# 		self.labBench.logBook.setStatusText = self.SetStatusText
+# 		# And tell the logBook to use our sink when the user wants to extract the log
+# 		self.labBench.logBook.log_sink = self.log_sink_to_file
 		# Override the shell so that status messages go to the status bar.
 		self.shell.setStatusText = self.SetStatusText
 		# make sure the std::cout & co are shown in the shell
@@ -247,55 +289,179 @@ class LabBook(EditorNotebookFrame):
 
 	def _modify_menus(self):
 		"""
-		Change a little the menus
+		Recreate the menus, overwritting what has been done in the
+		wx/py/frame.py
 		"""
+		art_prov = wx.ArtProvider()
+		# Remove predefined menus
+		for i in reversed(range(self.menuBar.GetMenuCount())):
+			self.menuBar.Remove(i)
 		# File Menu
-		m = self.fileMenu
-		m.SetTitle("Script")
-		ID_EXECUTE = wx.NewId()
-		m.Append(ID_EXECUTE, '&Execute \tCtrl+e',
+		m = self.fileMenu = wx.Menu()
+		m.Append(PyFrame.ID_NEW, '&New \tCtrl+N',
+				 'New file')
+		m.Append(PyFrame.ID_OPEN, '&Open... \tCtrl+O',
+				 'Open file')
+		m.AppendSeparator()
+		m.Append(PyFrame.ID_REVERT, '&Revert \tCtrl+R',
+				 'Revert to last saved version')
+		m.Append(PyFrame.ID_CLOSE, '&Close \tCtrl+W',
+				 'Close file')
+		m.AppendSeparator()
+		m.Append(PyFrame.ID_SAVE, '&Save... \tCtrl+S',
+				 'Save file')
+		m.Append(PyFrame.ID_SAVEAS, 'Save &As \tCtrl+Shift+S',
+				 'Save file with new name')
+		m.AppendSeparator()
+		m.Append(PyFrame.ID_PRINT, '&Print... \tCtrl+P',
+				 'Print file')
+		m.AppendSeparator()
+		m.Append(ID_HISTORY_TO_SCRIPT, 'Session &To Script',
+				 "Create a script from the session's history")
+		m.Append(ID_LOAD_OLD_SESSION, "Recover &Older Session",
+				 'Load (and exectute) the history of a previous session')		
+		m.AppendSeparator()
+		m.Append(ID_EXECUTE, '&Execute \tF9',
 				 'Execute the script in a new shell')
-		ID_EXECUTE_MAIN = wx.NewId()
-		m.Append(ID_EXECUTE_MAIN, '&Execute in main shell \tShift+Ctrl+e',
+		m.Append(ID_EXECUTE_MAIN, 'Execute in &main shell \tShift+F9',
 				 'Execute the script in the main shell')
+		m.Append(PyFrame.ID_NAMESPACE, '&Update Namespace \tCtrl+Shift+N',
+				 'Update namespace for autocompletion and calltips')
+		m.AppendSeparator()
+		m.Append(PyFrame.ID_EXIT, 'E&xit\tCtrl+Q', 'Exit Program')
 
-		# Options Menu
-		m = self.optionsMenu
-		m.SetTitle("Tools")
-		if wx.Platform == "__WXMAC__":
-			m.Remove(PyFrame.ID_USEAA)
-		# remove Py stuff that we don't want
-		m.Remove(m.FindItem("History"))
-		m.Remove(m.FindItem("Startup"))
-		m.Remove(m.FindItem("Settings"))
-		# add preference menu
-		ID_PREFERENCE = wx.NewId()
-		if wx.Platform == "__WXMAC__":
-			wx.App.SetMacPreferencesMenuItemId(ID_PREFERENCE)
-		else:
+		# Edit
+		m = self.editMenu = wx.Menu()
+		m.Append(PyFrame.ID_UNDO, '&Undo \tCtrl+Z',
+				 'Undo the last action')
+		m.Append(PyFrame.ID_REDO, '&Redo \tCtrl+Y',
+				 'Redo the last undone action')
+		m.AppendSeparator()
+		m.Append(PyFrame.ID_CUT, 'Cu&t \tCtrl+X',
+				 'Cut the selection')
+		m.Append(PyFrame.ID_COPY, '&Copy \tCtrl+C',
+				 'Copy the selection')
+		m.Append(PyFrame.ID_COPY_PLUS, 'Cop&y Plus \tCtrl+Shift+C',
+				 'Copy the selection - retaining prompts')
+		m.Append(PyFrame.ID_PASTE, '&Paste \tCtrl+V', 'Paste from clipboard')
+		m.Append(PyFrame.ID_PASTE_PLUS, "Past&e'n'run \tCtrl+Shift+V",
+				 'Paste and run commands')
+		m.AppendSeparator()
+		m.Append(PyFrame.ID_EMPTYBUFFER, 'E&mpty Buffer...',
+				 'Delete all the contents of the edit buffer')
+		m.Append(PyFrame.ID_SELECTALL, 'Select A&ll \tCtrl+A',
+				 'Select all text')
+		if wx.Platform == '__WXGTK__':
 			m.AppendSeparator()
-		m.Append(ID_PREFERENCE,"&Preferences", 
-				 "Open the preference panel")
+			m.Append(ID_PREFERENCE,"&Preferences", 
+					 "Open the preference panel")
 
-		# Help menu
-		m = self.helpMenu
-		ID_LIBRARY = wx.NewId()
-		m.Append(ID_LIBRARY,"&Library desk",
-				 "Show the library desk and its useful pointers")
-# 		ID_ABOUT = wx.NewId()
-# 		m.Append(ID_ABOUT,"&About", 
-# 				 "About this software")
-# 		if wx.Platform == "__WXMAC__":
-# 			wx.App.SetMacAboutMenuItemId(ID_ABOUT)
+		# View
+		m = self.autocompMenu = wx.Menu()
+		m.Append(PyFrame.ID_AUTOCOMP_SHOW, 'Show &Auto Completion\tCtrl+Shift+A',
+				 'Show auto completion list', wx.ITEM_CHECK)
+		m.Append(PyFrame.ID_AUTOCOMP_MAGIC, 'Include &Magic Attributes\tCtrl+Shift+M',
+				 'Include attributes visible to __getattr__ and __setattr__',
+				 wx.ITEM_CHECK)
+		m.Append(PyFrame.ID_AUTOCOMP_SINGLE, 'Include Single &Underscores\tCtrl+Shift+U',
+				 'Include attibutes prefixed by a single underscore', wx.ITEM_CHECK)
+		m.Append(PyFrame.ID_AUTOCOMP_DOUBLE, 'Include &Double Underscores\tCtrl+Shift+D',
+				 'Include attibutes prefixed by a double underscore', wx.ITEM_CHECK)
+		m = self.calltipsMenu = wx.Menu()
+		m.Append(PyFrame.ID_CALLTIPS_SHOW, 'Show Call &Tips\tCtrl+Shift+T',
+				 'Show call tips with argument signature and docstring', wx.ITEM_CHECK)
+		m.Append(PyFrame.ID_CALLTIPS_INSERT, '&Insert Call Tips\tCtrl+Shift+I',
+				 '&Insert Call Tips', wx.ITEM_CHECK)
+
+		m = self.viewMenu = wx.Menu()
+		m.AppendMenu(PyFrame.ID_AUTOCOMP, '&Auto Completion', self.autocompMenu,
+					 'Auto Completion Options')
+		m.AppendMenu(PyFrame.ID_CALLTIPS, '&Call Tips', self.calltipsMenu,
+					 'Call Tip Options')
+		m.Append(PyFrame.ID_WRAP, '&Wrap Lines\tCtrl+Shift+W',
+				 'Wrap lines at right edge', wx.ITEM_CHECK)
+		m.Append(PyFrame.ID_SHOW_LINENUMBERS, '&Show Line Numbers\tCtrl+Shift+L', 'Show Line Numbers', wx.ITEM_CHECK)
+		m.AppendSeparator()
+		m.Append(PyFrame.ID_TOGGLE_MAXIMIZE, '&Toggle Maximize\tF11', 'Maximize/Restore Application')
+		# add library menu
+		if wx.Platform == '__WXMAC__':
+			m.AppendSeparator()
+			m.Append(ID_LIBRARY,"&Library desk",
+					 "Show the library desk and its useful pointers")
+			m.Append(PyFrame.ID_HELP, '&Help\tF1', 'Help!')
+        		m.Append(PyFrame.ID_ABOUT, '&About...', 'About this program')
+		if wx.Platform != '__WXGTK__':
+			if wx.Platform == '__WXMAC__':
+				wx.App.SetMacPreferencesMenuItemId(ID_PREFERENCE)
+			m.Append(ID_PREFERENCE,"&Preferences", 
+					 "Open the preference panel")
+
+		# Search
+		m = self.searchMenu = wx.Menu()
+		m.Append(PyFrame.ID_FIND, '&Find Text... \tCtrl+F',
+				 'Search for text in the edit buffer')
+		m.Append(PyFrame.ID_FINDNEXT, 'Find &Next \tF3',
+				 'Find next/previous instance of the search text')
+
+		# Help menu (with a workaround for mac...)
+		m = self.helpMenu = wx.Menu()
+		m.Append(PyFrame.ID_HELP, '&Help\tF1', 'Help!')
+		if wx.Platform != "__WXMAC__":
+			m.AppendSeparator()
+			m.Append(ID_LIBRARY,"&Library desk",
+					 "Show the library desk and its useful pointers")
+			m.AppendSeparator()
+        		m.Append(PyFrame.ID_ABOUT, '&About...', 'About this program')
+
+		b = self.menuBar
+		b.Append(self.fileMenu, '&Script')
+		b.Append(self.editMenu, '&Edit')
+		b.Append(self.viewMenu, '&View')
+		b.Append(self.searchMenu, '&Search')
+		if wx.Platform != "__WXMAC__":
+			b.Append(self.helpMenu, '&Help')
 
 		# Bindings
 		self.Bind(wx.EVT_MENU, self.OnExecute, id=ID_EXECUTE)
 		self.Bind(wx.EVT_MENU, self.OnExecuteInMainShell, id=ID_EXECUTE_MAIN)
 		self.Bind(wx.EVT_MENU, self.OnPreference, id=ID_PREFERENCE)
-#   		self.Bind(wx.EVT_MENU, self.OnAbout, id=ID_ABOUT)
 		self.Bind(wx.EVT_MENU, self.OnLibrary, id=ID_LIBRARY)
-		self.Bind(wx.EVT_CLOSE, self.OnClose)
+		self.Bind(wx.EVT_MENU, self.OnHistoryToScript, id=ID_HISTORY_TO_SCRIPT)		
+ 		self.Bind(wx.EVT_MENU, self.OnLoadOldSession, id=ID_LOAD_OLD_SESSION)		
+		self.shell.Bind(wx.EVT_CLOSE, self.OnClose)		
 
+	def _checkCurrentBufferSaved(self,title="Savind required"):
+		"""
+		Check is the content of the current buffer has been saved (at
+		its latest version).
+		"""
+		if not self.bufferHasChanged():
+			return True
+		else:
+			d = wx.MessageDialog(self,u'You need to save the script first.\nDo you want to do is now ?',u'%s'%title, wx.YES_NO | wx.ICON_WARNING)
+			yes_no = d.ShowModal()
+			if yes_no != wx.ID_YES:
+				return False
+			cancel = self.bufferSave()
+			return not cancel
+
+	def _editorChange(self, editor):
+		"""
+		When the editor change, update the menu options.
+		"""
+		EditorNotebookFrame._editorChange(self,editor)
+		# enable/disable menu items that make sense
+		if isinstance(editor,wx.py.shell.Shell):
+			self.fileMenu.Enable(ID_EXECUTE,False)
+			self.fileMenu.Enable(ID_EXECUTE_MAIN,False)
+			self.fileMenu.Enable(ID_HISTORY_TO_SCRIPT,True)
+			self.fileMenu.Enable(ID_LOAD_OLD_SESSION,True)
+		else:
+			self.fileMenu.Enable(ID_EXECUTE,True)
+			self.fileMenu.Enable(ID_EXECUTE_MAIN,True)		
+			self.fileMenu.Enable(ID_HISTORY_TO_SCRIPT,False)
+			self.fileMenu.Enable(ID_LOAD_OLD_SESSION,False)
+	
 	def log_sink_to_file(self, text):
 		"""
 		Create a new file from the given text
@@ -303,20 +469,79 @@ class LabBook(EditorNotebookFrame):
 		self.bufferNew()		
 		self.editor.setText(text)
 
+	def bufferNew(self):
+		"""
+		Create new buffer.
+		
+		This method is redefined only to include a workaround for 
+		bug#1752674
+		"""
+		cancel = EditorNotebookFrame.bufferNew(self)
+		if not cancel:
+			self.editor.buffer.overwriteConfirm = lambda s,filepath=None: True
+			self.editor.buffer.confirmed = False
+		return cancel
+
+	def bufferPrint(self):
+		"""
+		printing ability
+		"""
+		printer = Printer()
+		if isinstance(self.editor,wx.py.shell.Shell):
+			printer.Print(self.editor.GetText(),
+						  "*Shell*")
+		else:
+			printer.Print(self.editor.getText(),
+						  self.editor.buffer.name)
+			
+	
 	def OnClose(self,event):
 		"""
 		Close everything and don't let a creepy plugin hold the end of the app.
 		"""
+		dispatcher.send(signal="LabBook.close")
 		sys.exit(0)
 
 	def OnHelp(self,event):
 		"""
 		Display a small helpign text
 		"""
-		msg = """Code samples and documentation are available at the library desk
-(accessible at the bottom of the application's window).
+		msg = """\
+Basic usage
+-----------
+
+  1) type and execute a few commands in the shell
+
+  2) ask for this commands to be transformed into a plain script file
+    (via the menu "Script"->"Session To script")
+
+  3) edit the script
+
+  4) execute it with Script->Execute
+
+  5) save it when you're done !
+
+and later on...
+
+  - open an existing script and edit it
+
+  - restart an old shell session (see "Script"->"Recover Old Session")
+
+Search for documentation
+------------------------
+
+Code samples and documentation are available at the library desk:
+
+  - type a query at the bottom of the application's window
+
+or
+
+  - launch the desk via the menu "Help"->"Library Desk"
+
+
+Shortcut keys
+-------------
 
-Shortcut keys for the shell are listed below: 
 (also accessible by typing shell.help())
 
 %s
@@ -351,6 +576,8 @@ And heavily based on wxPython's py library.
 		"""
 		Execute the script in a new shell frame.
 		"""
+		if not self._checkCurrentBufferSaved():
+			return
 		script_filepath = self.editor.buffer.doc.filepath
 		if self.execution_shells.has_key(script_filepath):
 			execframe = self.execution_shells[script_filepath]
@@ -370,13 +597,16 @@ And heavily based on wxPython's py library.
 		# amke sure the window is visible
 		execframe.Hide()
 		execframe.Show()
-		# exec the file (maybe there is a cleaner way to do this ?)
-		execframe.shell.run("execfile('%s')" % script_filepath)
+		# exec the file
+#		execframe.shell.run("execfile('%s')" % script_filepath)
+		execframe.shell.runfile(script_filepath)
 
 	def OnExecuteInMainShell(self,event):
 		"""
 		Execute the script in the main shell.
 		"""
+		if not self._checkCurrentBufferSaved():
+			return
 		filepath = self.editor.buffer.doc.filepath
 		self.notebook.SetSelection(0)
 		# exec the file (maybe there is a cleaner way to do this ?)
@@ -395,7 +625,40 @@ And heavily based on wxPython's py library.
 		else:
 			self.options_frame.Hide()
 			self.options_frame.Show()
+
+
+	def OnHistoryToScript(self,event):
+		"""
+		Pass the shell's history to a new script.
+		"""
+		txtL = self.historyman.get_history_from_landmark()
+		txtL.reverse()
+		script = os.linesep.join(txtL)
+		return self.log_sink_to_file(script)
 		
+		
+	def OnLoadOldSession(self,event):
+		"""
+		Load an older session from its history file.
+		"""
+		d = wx.FileDialog(self,"Please chose a session to load",self.history_dir,
+						  wildcard="*.session", style=wx.FD_OPEN)
+		ok_cancel = d.ShowModal()
+		path = d.GetPath()
+		d.Destroy()
+		if ok_cancel==wx.ID_OK and os.path.isfile(path):
+			f = open(path)
+			txt = f.read()
+			txtL = txt.split(self.historyman.COMMANDS_SEPARATOR)
+			txtL.reverse()
+# 			print txtL
+			sesname = os.path.splitext(os.path.basename(path))[0]
+			self.shell.run("# Recovering session: %s" % sesname)
+#   			for c in txtL:
+#  				self.shell.run(c)
+  			self.shell.Execute("\n\n".join(txtL))
+
+
 	def _onOptionsChanged(self):
 		"""
 		Take into accout a change in the options
